import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"

import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Шаблон диалога выбора записи из списка
 * @param model модель данных
 * @param titleTr текст загаловка диалога
 * @param okButtonTr текст кнопки выбора позиции
 * @param cancelButtonTr текст отмены диалога
 * @param inputPlaceholderTr текст пустого поля input
 * @param columnsHorizontalAlignment выравнивание текста в колонках.
                                     Размер массива должен совпадать с числом столбцов
                                     модели данных или размером columnsOrder
 * @param columnsWidth ширина колонок.
                       Размер массива должен совпадать с числом столбцов модели данных
                       или размером columnsOrder.
 * @param columnsOrder список названий столбцов в требуемом порядке.
                       При пустом значении сохроняется оригинальный порядок модель данных.
                       Столбцы, не указанные в непустом списке, будут скрыты.
 * @param filter фильтр для модели данных
 * @param tableAvailableWidth ширина для контента таблицы данных
 */
FullscreenDialog {
    id: root

    required property AbstractItemModel model
    property var titleTr
    required property var okButtonTr
    required property var cancelButtonTr
    required property var inputPlaceholderTr
    required property list<int> columnsHorizontalAlignment
    required property list<int> columnsWidth
    property list<string> columnsOrder: []
    property alias filter: inputField.text
    readonly property real tableAvailableWidth: tableView.width - 48 // 48 это отступы внутри row

    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter { onCancel: root.reject() }

    /**
     * @brief Сигнал выбора записи из таблицы
     * @param index индекс выбранной ячейки
     */
    signal accept(var index)
    /**
     * @brief Сигнал закрытия диалога без выбора записи
     */
    signal reject()
    /**
     * @brief Выбрать выделенную запись
     */
    function selectCurrentIndex() {
        if (selectionModel.hasSelection)
            accept(proxyModel.mapToSource(selectionModel.currentIndex))
    }

    onFilterChanged: tableView.contentY = 0

    ItemSelectionModel {
        id: selectionModel
        model: ColumnSortingProxyModel {
            id: proxyModel
            sourceModel: root.model
            columnsOrder: root.columnsOrder
        }
    }

    contentItem: RowLayout {
        spacing: 0
        // Таблица
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                BasicLabel {
                    Style.class: "choiceTableMessage"
                    padding: 24
                    Tr { tr: root.titleTr }
                }
                HorizontalHeaderView {
                    id: headerView
                    required property var model
                    Layout.fillWidth: true
                    Layout.preferredHeight: 30
                    model: proxyModel
                    syncView: tableView
                    clip: true
                    resizableColumns: false
                    rowHeightProvider: function(row) { return 30 }
                    interactive: false
                    delegate: BasicLabel {
                        Style.class: "choiceTableHeaderDelegate"
                        padding: 0
                        leftPadding: column === 0 ? 24 : 0
                        rightPadding: column === tableView.columns - 1 ? 24 : 0
                        verticalAlignment: Label.AlignVCenter
                        horizontalAlignment: root.columnsHorizontalAlignment[column]
                        Tr { tr: decoration }

                        // FIXME: модель шлет сигнал modelReset, из-за чего виджет пересоздается.
                        // Но после удаления старый виджет продолжает отображаться на экране, причем в неправильной позиции.
                        // Из-за этого возникает задвоение заголовков. Установка `visible = false` при удалении - костыль
                        Component.onDestruction: () => { visible = false }
                    }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: strokeWidth
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    signal rowPressedChanged(int row, bool value)
                    model: proxyModel
                    clip: true
                    contentWidth: width
                    columnWidthProvider: function(column) {
                        if (column === 0 || column === columns - 1)
                            return root.columnsWidth[column] + 24
                        return root.columnsWidth[column]
                    }
                    rowHeightProvider: function(row) { return 68 }
                    rowSpacing: 1
                    resizableColumns: false

                    // При масштабировании, списки могут отображаться с артефактами.
                    // Рендерим с мультисемплированием в нужном разрешении. Не знаю за счет чего, но проблему это решило.
                    layer {
                        enabled: mainWindow.multisampling
                        samples: 2
                        textureSize: Qt.size(width * mainWindow.scaleFactor, height * mainWindow.scaleFactor)
                    }

                    delegate: BasicButton {
                        id: tableDelegate
                        Style.class: "choiceTableRowDelegate"
                        text: display
                        verticalAlignment: Qt.AlignVCenter
                        horizontalAlignment: root.columnsHorizontalAlignment[column]
                        padding: 0
                        leftPadding: column === 0 ? 24 : 0
                        rightPadding: column === tableView.columns - 1 ? 24 : 0
                        textLayout { fillWidth: true; fillHeight: true }
                        down: pressed || selectionModel.currentIndex.row === row
                        pressed: false
                        Line {
                            anchors.top: parent.bottom
                            width: tableDelegate.width
                            strokeWidth: 1
                            color: Style.parameter("color.grayBorder")
                        }
                        MouseArea {
                            id: ma
                            anchors.fill: parent
                            onPressedChanged: tableView.rowPressedChanged(row, pressed)
                            onClicked: selectionModel.setCurrentIndex(proxyModel.index(row, column), ItemSelectionModel.SelectCurrent) // выделить позицию
                        }
                        Connections {
                            target: tableView
                            function onRowPressedChanged(row, pressed) {
                                if (row === model.row)
                                    tableDelegate.pressed = pressed
                            }
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        // Цифровая клавиатура
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 0
                BasicTextField {
                    id: inputField
                    Style.class: "choiceTableInput"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 64
                    padding: 12
                    hideCursor: true
                    fixedCursorPosition: true
                    Tr on placeholderText { tr: root.inputPlaceholderTr }
                }
                Item { Layout.preferredHeight: 24 }
                InlineKeyboard {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 296
                    layoutType: "digits"
                }
                Item { Layout.preferredHeight: 16 }
                BasicButton {
                    Style.class: "choiceTableCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { tr: root.cancelButtonTr }
                    onClicked: root.reject()
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "choiceTableOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { tr: root.okButtonTr }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: root.selectCurrentIndex()
                }
            }
        }
    }
}
